<?php

class FTP {
	private static $ftp_user_name = 'root';
	private static $ftp_user_pass = 'root';

    // IP address of last successful connection.
    // This variable has no meaning unless connection_id !== FALSE.
	private static $last_connect_ip = "";
	// ID of the active connection.
	// If this variable === FALSE, there is no active connection.
	private static $connection_id = FALSE;

	private static function connect_if_needed($ip_address)
	{
		if(self::$last_connect_ip != $ip_address)
		{
			if(self::$last_connect_ip != "")
			{
				ftp_close(self::$connection_id);
			}
			self::$connection_id = ftp_connect($ip_address, '21', '5');
			if(self::$connection_id === FALSE)
			{
				//throw new Exception("FTP connection failure (" . $ip_address . ")");
				//die("Connection to \"" . $ip_address . "\" failed. Is the device connected to the network?");
				
				// This device is not responding - remove it from the online device cache. 
				require_once("models/mymix_discover.php");
				MyMix_Discover::remove_device_from_cache($ip_address);
				return;
			}
			self::$last_connect_ip = $ip_address;
			$login_result = ftp_login(self::$connection_id, self::$ftp_user_name, self::$ftp_user_pass);
		}
	}
	
	public static function disconnect()
	{
	    if(self::is_connected())
		{
			ftp_close(self::$connection_id);
			self::$last_connect_ip = "";
			self::$connection_id = FALSE;
		}
	}
	
	// Returns true if there is an active connection
	public static function is_connected()
	{
	    return (self::$connection_id !== FALSE);
	}

	private static function put_file($local_path, $ip_address, $remote_path)
	{
		self::connect_if_needed($ip_address);
		// If connection fails, don't try to push files
		if(!self::is_connected())
		{
		    return;
		}
		$success = ftp_put(self::$connection_id, $remote_path, $local_path, FTP_BINARY);
		if(!$success)
		{
			/*throw new Exception("FTP PUT failed".
				" IP: " . $ip_address .
				" remote path: " . $remote_path);*/
			die("FTP PUT failed".
				" IP: " . $ip_address .
				" remote path: " . $remote_path);
		}
	}

	private static function get_file($local_path, $ip_address, $remote_path)
	{
		// create the local directory if it doesn't exist
		self::mkdir_if_needed(dirname($local_path));

		self::connect_if_needed($ip_address);
		// If connection fails, don't try to pull files
		if(!self::is_connected())
		{
		    return;
		}
		
		$success = ftp_get(self::$connection_id, $local_path, $remote_path, FTP_BINARY);
		if(!$success)
		{
			/*throw new Exception("FTP GET failed".
				" IP: " . $ip_address .
				" remote path: " . $remote_path);*/
			die("FTP GET failed".
				" IP: " . $ip_address .
				" remote path: " . $remote_path);
		}
	}

	private static function mkdir_if_needed($path)
	{
		if(!is_dir($path))
		{
			mkdir($path, 0777, TRUE);
		}
	}

	public static function push_mymix_profile($ip_address, $mac_address, $profile_name)
	{
		require_once("helpers/config.php");
		self::put_file(
			Config::get_item('mymix_devices_dir') . $mac_address . "/profiles/" . $profile_name,
			$ip_address,
			"/mnt/settings/profiles/" . $profile_name
		);
	}

	public static function pull_mymix_profile($ip_address, $mac_address, $profile_name)
	{
		require_once("helpers/config.php");
		self::get_file(
			Config::get_item('mymix_devices_dir') . $mac_address . "/profiles/" . $profile_name,
			$ip_address,
			"/mnt/settings/profiles/" . $profile_name
		);
	}

	public static function push_all_mymix_config($ip_address, $mac_address)
	{
		require_once("models/mymix_profile.php");
		
		self::connect_if_needed($ip_address);
		// If connection fails, don't try to push files
		if(!self::is_connected())
		{
		    return;
		}

		// Push the profiles first, in case one was renamed or a new one created.
		$local_profiles = MyMix_Profile::get_profile_list($mac_address);
		foreach($local_profiles as $this_profile)
		{
			self::push_mymix_profile($ip_address, $mac_address, $this_profile);
		}

		// Inotify on the myMix gets events every 500ms. Give the myMix some
		// time to make sure it "sees" the profiles change.
		// Without this delay, the myMix will re-save the prefs, after
		// we update it below.
		sleep(1);

		// Push the preferences second, so that the current profile will be correct.
		self::push_mymix_preference($ip_address, $mac_address);
		
		// Inotify on the myMix gets events every 500ms. Give the myMix some
		// time to make sure it "sees" the prefs change.
		// Without this delay, the myMix will re-save the current profile, after
		// we (possibly) delete it below.
		sleep(1);

		// Lastly, delete any profiles that were deleted or renamed locally. This needs
		// to be done last so that we don't delete the MyMix's current profile.
		$remote_profiles = self::get_mymix_profile_list($ip_address);
		foreach($remote_profiles as $this_profile)
		{
			if(!in_array($this_profile, $local_profiles))
			{
				self::delete_mymix_profile($ip_address, $this_profile);
			}
		}

		// Push the lock settings last - order doesn't appear to be important for locks.
		self::push_mymix_lock($ip_address, $mac_address);
		
		self::push_mymix_runtime($ip_address, $mac_address);
	}

	public static function pull_all_mymix_config($ip_address, $mac_address)
	{
		require_once("models/mymix_profile.php");
		
		self::connect_if_needed($ip_address);
		// If connection fails, don't try to pull files
		if(!self::is_connected())
		{
		    return;
		}

		// Pull the preferences
		self::pull_mymix_preference($ip_address, $mac_address);

		// Pull the profiles
		$remote_profiles = self::get_mymix_profile_list($ip_address);
		foreach($remote_profiles as $profile_name)
		{
			self::pull_mymix_profile($ip_address, $mac_address, $profile_name);
		}

		// Delete any local profiles that aren't on the myMix
		$local_profiles = MyMix_Profile::get_profile_list($mac_address);
		foreach($local_profiles as $this_profile)
		{
			if(!in_array($this_profile, $remote_profiles))
			{
				MyMix_Profile::delete_profile($mac_address, $this_profile);
			}
		}

		// Pull lock settings
		self::pull_mymix_lock($ip_address, $mac_address);
		
		self::pull_mymix_runtime($ip_address, $mac_address);
	}

	public static function push_iex16_profile($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		self::put_file(
			Config::get_item('iex16_devices_dir') . $mac_address . "/prefs",
			$ip_address,
			"/mnt/settings/prefs"
		);
	}

	public static function pull_iex16_profile($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		self::get_file(
			Config::get_item('iex16_devices_dir') . $mac_address . "/prefs",
			$ip_address,
			"/mnt/settings/prefs"
		);
	}

	public static function push_mymix_preference($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		self::put_file(
			Config::get_item('mymix_devices_dir') . $mac_address . "/prefs",
			$ip_address,
			"/mnt/settings/prefs"
		);
	}

	public static function pull_mymix_preference($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		self::get_file(
			Config::get_item('mymix_devices_dir') . $mac_address . "/prefs",
			$ip_address,
			"/mnt/settings/prefs"
		);
	}

	public static function get_mymix_profile_list($ip_address)
	{
		$retval = array();
		
		self::connect_if_needed($ip_address);
		// If connection fails, don't try to do FTP commands
		if(!self::is_connected())
		{
		    return;
		}
		
		$list = ftp_nlist(self::$connection_id, "/mnt/settings/profiles");
		if($list !== FALSE)
		{
			foreach($list as $this_profile)
			{
				$retval[] = basename($this_profile);
			}
		}

		return $retval;
	}

	public static function delete_mymix_profile($ip_address, $profile_name)
	{
		if(empty($ip_address) || empty($profile_name))
		{
			//throw new Exception("Tried to delete a profile without an IP address or profile name!");
			die("Tried to delete a profile without an IP address or profile name!");
		}

		self::connect_if_needed($ip_address);
		// If connection fails, don't try to pull files
		if(!self::is_connected())
		{
		    return;
		}
		
		$remote_path = "/mnt/settings/profiles/" . $profile_name;
		$success = ftp_delete(self::$connection_id, $remote_path);
		if(!$success)
		{
			/*throw new Exception("FTP DELETE failed".
				" IP: " . $ip_address .
				" remote path: " . $remote_path);*/
			die("FTP DELETE failed".
				" IP: " . $ip_address .
				" remote path: " . $remote_path);
		}
	}

	public static function rename_mymix_profile($ip_address, $old_profile_name, $new_profile_name)
	{
		if(empty($ip_address) || empty($old_profile_name) || empty($new_profile_name))
		{
			//throw new Exception("Tried to rename a profile without an IP address or profile name!");
			die("Tried to rename a profile without an IP address or profile name!");
		}
		
		self::connect_if_needed($ip_address);
		// If connection fails, don't try to pull files
		if(!self::is_connected())
		{
		    return;
		}
		
		$old_path = "/mnt/settings/profiles/" . $old_profile_name;
		$new_path = "/mnt/settings/profiles/" . $new_profile_name;
		$success = ftp_rename(self::$connection_id, $old_path, $new_path);
		if(!$success)
		{
			/*throw new Exception("FTP RENAME failed".
				" IP: " . $ip_address .
				" original path: " . $old_path .
				" new path: " . $new_path);*/
			die("FTP RENAME failed".
				" IP: " . $ip_address .
				" original path: " . $old_path .
				" new path: " . $new_path);
		}
	}

	public static function push_mymix_lock($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		self::put_file(
			Config::get_item('mymix_devices_dir') . $mac_address . "/menulock/menulock_mgr",
			$ip_address,
			"/mnt/settings/menulock/menulock_mgr"
		);
	}

	public static function pull_mymix_lock($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		$local_path = Config::get_item('mymix_devices_dir') . $mac_address . "/menulock/menulock_mgr";
		self::mkdir_if_needed(dirname($local_path));
		self::get_file($local_path, $ip_address, "/mnt/settings/menulock/menulock_mgr");
	}
	
	public static function push_mymix_runtime($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		self::put_file(
			Config::get_item('mymix_devices_dir') . $mac_address . "/runtime",
			$ip_address,
			"/tmp/runtime/writeable"
		);
	}

	public static function pull_mymix_runtime($ip_address, $mac_address)
	{
		require_once("helpers/config.php");
		$local_path = Config::get_item('mymix_devices_dir') . $mac_address . "/runtime";
		self::mkdir_if_needed(dirname($local_path));
		self::get_file($local_path, $ip_address, "/tmp/runtime/readable");
	}
}

/* End of ftp.php*/
