<?php

class MyMix_Logic
{
	private static $local_mac_for_compare_channels;

	// Returns a list of channels with 'online' and 'active selected' data
	public static function get_channels_with_extra_data($mac, $profile_name, $prefs, $profile, $include_local_inputs)
	{
		require_once("models/iex16_profile.php");
		require_once("models/mymix_discover.php");
		require_once("helpers/ftp.php");
		require_once("helpers/validate.php");

		// array to track which devices have been synced
		$synced_devices = array($mac);

		// keep track of the "active" selected channels
		$active_selected = 0;

		// Local channels use 2 "active selected" channels if they are active in any mode.
		if(!$include_local_inputs && $profile->get_input_mode() != "none")
		{
			$active_selected += 2;
		}

		$channels = array();
		for($channel_num = 0; $channel_num < $profile->get_remote_channel_count(); $channel_num++)
		{
			// Make sure MAC address is properly formed before using it.
			// MAC addresses stored in the remote channels section of the myMix
			// profile sometimes leave off leading zeros.
			$formatted_mac = Validate::format_mac_address($profile->get_remote_channel_mac($channel_num));

			// Remove local channels from list
			if(!$include_local_inputs && $formatted_mac == $mac)
			{
				continue;
			}

			$this_channel['unit_name'] = $profile->get_remote_channel_unit_name($channel_num);
			$this_channel['remote_channel_name'] = $profile->get_remote_channel_name($channel_num);
			$this_channel['selected'] = ($profile->get_remote_channel_selected($channel_num) == 1);
			$this_channel['remote_channel_volume'] = $profile->get_remote_channel_volume($channel_num);
			$this_channel['remote_channel_tone'] = $profile->get_remote_channel_tone($channel_num);
			$this_channel['remote_channel_effect_send'] = $profile->get_remote_channel_effect_send($channel_num);
			$this_channel['remote_channel_pan'] = $profile->get_remote_channel_pan($channel_num);
			$this_channel['remote_channel_solo'] = ($profile->get_remote_channel_solo($channel_num) == "on");
			$this_channel['remote_channel_mute'] = ($profile->get_remote_channel_mute($channel_num) == "on");
			$this_channel['id'] = $channel_num;
			$this_channel['remote_channel_input_mode'] = $profile->get_remote_channel_input_mode($channel_num);
			$this_channel['remote_channel_mac'] = $formatted_mac;
			$this_channel['online'] = FALSE;

            // Attempt to sync the device
   			$this_device = MyMix_Discover::get_device_by_mac("all", $formatted_mac);
			if(!empty($this_device))
			{
				// found the device

				// sync the device if it hasn't been already
				if(!in_array($this_device['mac'], $synced_devices))
				{
					$synced_devices[] = $this_device['mac'];
					if($this_device['type'] == "mymix")
					{
						FTP::pull_mymix_preference($this_device['ip_address'], $this_device['mac']);
					    $remote_device_prefs = new MyMix_Preference(Config::get_item('mymix_devices_dir') .
							$this_device['mac'] . "/prefs");
						// only pull the current profile - no need to pull the rest for this page
						FTP::pull_mymix_profile($this_device['ip_address'],
							$this_device['mac'],
							$remote_device_prefs->get_current_profile());
					}
					else
					{
						FTP::pull_iex16_profile($this_device['ip_address'], $this_device['mac']);
					}
				}
			}

            // Now that we've attempted to sync the device, check if it's still considered 'online'.
            $this_device = MyMix_Discover::get_device_by_mac("all", $formatted_mac);
		    if(!empty($this_device))
		    {
			    // Check if this channel is online.
			    if($this_device['type'] == "mymix")
			    {
				    $this_channel['online'] = self::is_mymix_channel_online(
					    $this_device['mac'],
					    $this_channel['remote_channel_name'],
					    $this_channel['remote_channel_input_mode']
				    );
			    }
			    else
			    {
				    $this_channel['online'] = self::is_iex16_channel_online(
					    $this_device['mac'],
					    $this_channel['remote_channel_name'],
					    $this_channel['remote_channel_input_mode']
				    );
				
			    }
		    }

		    // Only 16 selected channels are allowed to be "active"
		    if($this_channel['online'] && 
			    $this_channel['selected'] && 
			    $active_selected < 16)
		    {
			    $this_channel['active_selected'] = TRUE;
			    $active_selected++;
			    if($this_channel['remote_channel_input_mode'] == "stereo" ||
				    $this_channel['remote_channel_input_mode'] == "mono")
			    {
				    $active_selected++;
			    }
		    }
		    else
		    {
			    $this_channel['active_selected'] = FALSE;
		    }

		    $channels[$channel_num] = $this_channel;
	    }

		// Used to sort local inputs to the top of the list
		self::$local_mac_for_compare_channels = $mac;
		usort($channels, array("MyMix_Logic", "compare_channels"));

		return $channels;
	}

	// used by usort()
	private static function compare_channels($a, $b)
	{
		$retval = 0;

		if($a['active_selected'] && !$b['active_selected'])
		{
			$retval = -1;
		}
		elseif(!$a['active_selected'] && $b['active_selected'])
		{
			$retval = 1;
		}

		if($retval == 0)
		{
			if($a['online'] && !$b['online'])
			{
				$retval = -1;
			}
			elseif(!$a['online'] && $b['online'])
			{
				$retval = 1;
			}
		}

		if($retval == 0)
		{
			if($a['remote_channel_mac'] == self::$local_mac_for_compare_channels && 
				$b['remote_channel_mac'] != self::$local_mac_for_compare_channels)
			{
				$retval = -1;
			}
			elseif($a['remote_channel_mac'] != self::$local_mac_for_compare_channels &&
				$b['remote_channel_mac'] == self::$local_mac_for_compare_channels)
			{
				$retval = 1;
			}
		}

		if($retval == 0)
		{
			$retval = strcmp($a['unit_name'], $b['unit_name']);
		}

		if($retval == 0)
		{
			$retval = strcmp($a['remote_channel_name'], $b['remote_channel_name']);
		}
		
		return $retval;
	}

	private static function is_mymix_channel_online($mymix_mac, $channel_name, $input_mode)
	{
		$remote_device_prefs = new MyMix_Preference(Config::get_item('mymix_devices_dir') .
			$mymix_mac . "/prefs");
		$remote_device_profile = new MyMix_Profile(Config::get_item('mymix_devices_dir') .
			$mymix_mac . "/profiles/" . $remote_device_prefs->get_current_profile());

		// Take "transmit to network" setting into account.
		if($channel_name == "Stereo Mix" && 
			$remote_device_profile->get_transmit_stereo_mix() == "on")
		{
			// found the device and channel
			return TRUE;
		}
		// Input mode has to match and can't be 'none'
		elseif($remote_device_profile->get_input_mode() == $input_mode &&
				$remote_device_profile->get_input_mode() != "none")
		{
			// If the input mode is stereo or mono, only compare channel 1.
			if($input_mode == "stereo" || $input_mode == "mono")
			{
				if($remote_device_profile->get_ch1_name() == $channel_name)
				{
					// found the device and channel
					return TRUE;
				}
			}
			else
			{
				// Input mode must be dual mono.
				// Check both remote channel names.
				if(	($remote_device_profile->get_ch1_name() == $channel_name ||
					$remote_device_profile->get_ch2_name() == $channel_name )
				)
				{
					// found the device and channel
					return TRUE;
				}
			}
		}

		return FALSE;
	}

	private static function is_iex16_channel_online($iex16_mac, $channel_name, $input_mode)
	{
		$remote_device_prefs = new IEX16_Profile(Config::get_item('iex16_devices_dir') .
			$iex16_mac . "/prefs");
		for($prefs_chan = 0; $prefs_chan < 16; $prefs_chan++)
		{
			// Mimic myMix's IEX16 channel renaming.
			// See bugzilla bug #69.
			$modified_channel_name = self::iex16_channel_rename(
				$remote_device_prefs->get_channel_name($prefs_chan), $prefs_chan);

			// Convert IEX16 input mode values to those stored in the remotechannels section.
			$modified_input_mode = "";
			switch($remote_device_prefs->get_channel_input_mode($prefs_chan))
			{
				case "N":
					$modified_input_mode = "none";
					break;
				case "DM":
					$modified_input_mode = "dual";
					break;
				case "S":
					$modified_input_mode = "stereo";
					break;
			}

			// Match channels on name and input mode.
			// Channels with input mode of none are considered offline.
			if($modified_channel_name == $channel_name &&
				$modified_input_mode == $input_mode &&
				$modified_input_mode != "none"
			)
			{
				// This channel is online
				return TRUE;
			}
		}

		return FALSE;
	}

	// If the myMix is online, this function asks it to save out its current
	// profile updated with the latest remote channel list.
	public static function sync_profile_with_channels($mac, $profile_name)
	{
		$this_device = MyMix_Discover::get_device_by_mac("mymix", $mac);
		if(!empty($this_device))
		{
			// Found the device

			// Tell myMix to write the current profile to flash
			FTP::pull_mymix_preference($this_device['ip_address'], $mac);
			FTP::push_mymix_preference($this_device['ip_address'], $mac);
	
			// Inotify on the myMix gets events every 500ms. Give the myMix some
			// time to make sure it "sees" the prefs file had been rewritten.
			// Without this delay, the myMix may not write out the profile with 
			// channels before we pull it below.
			sleep(1);

			// Pull profile from it
			FTP::pull_mymix_profile($this_device['ip_address'], $mac, $profile_name);
		}
	}

	// myMix adds zero-padded channel number to IEX16 channels, if they don't already have it.
	// This function mimics the myMix's renaming behavior.
	public static function iex16_channel_rename($channel_name, $channel_index)
	{
		$modified_channel_name = $channel_name;
		$num_check = substr($modified_channel_name, 0, 2);
		if(!is_numeric($num_check) || (int)$num_check != ($channel_index + 1))
		{
			$modified_channel_name = sprintf("%02d %s", 
				($channel_index + 1), 
				$channel_name);
		}
		return $modified_channel_name;
	}
}

/* End of mymix_logic.php */
