<?php

class MyMix_Runtime
{
    private static $default_xml = <<<XML
<?xml version="1.0" encoding="iso-8859-1"?>
<runtime>
    <watchlist>
        <watch index="0">
            <name>MUTE</name>
            <write>1</write>
            <type>bool</type>
            <value>0</value>
        </watch>
    </watchlist>
</runtime>
XML;

	private $xml_data;
	private $xml_path;
	
    function __construct($filename)
	{
		$this->load($filename);
	}

    // Returns a SimpleXMLElement that refers to the base <runtime> element
	// and contains a structure like that of default_xml, above.
	private function load($filename)
	{
		//libxml_use_internal_errors(true);
		$retval = simplexml_load_file($filename, "SimpleXMLElement", LIBXML_NOEMPTYTAG);
		if($retval === FALSE)
		{
			/*$errors = libxml_get_errors();
			$error_message = "";
			foreach ($errors as $error)
			{
				$error_message .= $error->message . " in " . $error->file . " at line " . $error->line . ".\n";
			}

			throw new Exception("Failed to open file: " . $error_message);*/
			$retval = new SimpleXMLElement(self::$default_xml);
		}

		$this->xml_data = $retval;
		$this->xml_path = $filename;
	}
		
	// Writes profile out to filename.
	public function save()
	{
		require_once("helpers/xml.php");

		$path = dirname($this->xml_path);
		if(!is_dir($path))
		{
			mkdir($path, 0777, TRUE);
		}

		$dom = dom_import_simplexml($this->xml_data);
		$temp_output = $dom->ownerDocument->saveXML($dom->ownerDocument, LIBXML_NOEMPTYTAG);
		$temp_output = XML::pretty_format($temp_output);
		$result = file_put_contents($this->xml_path, $temp_output);
		if($result === FALSE)
		{
			$error = error_get_last();
			throw new Exception("Failed to open file: " . $error['message']);
		}
	}
	
	private function bool_to_string($bool)
	{
		if($bool === TRUE || $bool == "true" || $bool == "1")
		{
			return "1";
		}
		else
		{
			return "0";
		}
	}
	
	public function set_master_mute($value)
	{
	    $value = $this->bool_to_string($value);
		$this->xml_data->watchlist->watch[0]->value = $value;
	}

	public function get_master_mute()
	{
		return (string)$this->xml_data->watchlist->watch[0]->value;
	}
}

/* End of mymix_runtime.php */
