<?php

class Users {

    private static $default_xml = <<<XML
<?xml version="1.0" encoding="iso-8859-1"?>
<users>
    <user>
        <name>admin</name>
        <password>password</password>
        <userclass>admin</userclass>
    </user>
    <user>
        <name>user</name>
        <password>password</password>
        <userclass>user</userclass>
    </user>
</users>
XML;

	// Returns TRUE if username is a valid user and password is
	// correct, or FALSE otherwise.
	// NOTE: This function will return FALSE if the user info file is missing.
	public static function authenticate($username, $password)
	{
		$user_info = self::get_user_info($username);

		if(empty($user_info))
		{
			return FALSE; // Didn't find user
		}

		if($user_info['password'] == $password)
		{
			return TRUE; // Found and authenticated user
		}
		return FALSE; // Found user but incorrect password
	}

	public static function is_admin($username)
	{
		$user_info = self::get_user_info($username);
		if(empty($user_info))
		{
			return FALSE; // Didn't find user
		}

		if($user_info['userclass'] == "admin")
		{
			return TRUE; // Found user and it is an admin
		}
		return FALSE; // Found user but it is not an admin
	}

	private static function get_user_info($username)
	{
		require_once("helpers/config.php");

		$retval = array();

        $users_xml = self::get_users_xml();
        $foundit = FALSE;
		foreach($users_xml as $this_user)
		{
			if($this_user->name == $username)
			{
				$retval['password'] = (string)$this_user->password;
				$retval['userclass'] = (string)$this_user->userclass;
				$foundit = TRUE;
				break;
			}
		}
		
		return $retval;
	}

	// Returns TRUE on success, FALSE otherwise
	public static function change_password($user, $password)
	{
		require_once("helpers/config.php");

		$retval = FALSE;

		$users_xml = self::get_users_xml();

		foreach($users_xml as $this_user)
		{
			if($this_user->name == $user)
			{
				$this_user->password = $password;
				$users_xml->saveXML(Config::get_item('user_info_file'));
				$retval = TRUE;
				break;
			}
		}
		
		// Restore the admin or user password entries if they get removed
		if(!$retval && ($user == "admin" || $user == "user") )
		{
		    $this_user = $users_xml->addChild("user");
		    $this_user->addChild("name", $user);
		    $this_user->addChild("password", $password);
		    if($user == "admin")
		    {
		        $this_user->addChild("userclass", "admin");
		    }
		    else
		    {
		        $this_user->addChild("userclass", "user");
		    }
		    $users_xml->saveXML(Config::get_item('user_info_file'));
		    $retval = TRUE;
		}

		return $retval;
	}
	
	private static function get_users_xml()
	{
	    // Load the XML file
		$retval = FALSE;
		if(file_exists(Config::get_item('user_info_file')))
		{
	    	$retval = simplexml_load_file(Config::get_item('user_info_file'));
		}
	    
	    // If the file cannot be loaded, use the default XML
	    if($retval === FALSE)
	    {
	        $retval = new SimpleXMLElement(self::$default_xml);
	    }
	    
	    return $retval;
	}
}

/* End of users.php */
